from autohack.core.path import *
import subprocess, threading, psutil, time, os


class CodeRunner:
    class Result:
        def __init__(
            self,
            timeOut: bool,
            memoryOut: bool,
            returnCode: int | None,
            stdout: bytes | None,
            stderr: bytes | None,
        ) -> None:
            self.timeOut = timeOut
            self.memoryOut = memoryOut
            self.returnCode = returnCode
            self.stdout = stdout
            self.stderr = stderr

    def __init__(self):
        self.timeOut = False
        self.memoryOut = False

    def memoryMonitor(
        self, pid: int, timeLimit: float | None, memoryLimit: int | None
    ) -> None:
        try:
            psutilProcess = psutil.Process(pid)
        except psutil.NoSuchProcess:
            # 跑的太他妈快了，没测到
            return
        startTime = psutilProcess.create_time()
        while True:
            try:
                # psutilProcess.cpu_times();
                if timeLimit is not None and time.time() - startTime > timeLimit:
                    self.timeOut = True
                    psutilProcess.kill()
                    return
                # 测出来是资源监视器内存中提交那栏 *1024
                if (
                    memoryLimit is not None
                    and psutilProcess.memory_info().vms > memoryLimit
                ):
                    self.memoryOut = True
                    psutilProcess.kill()
                    return
            except psutil.NoSuchProcess:
                return

    def run(
        self,
        *popenargs,
        inputContent: bytes | None = None,
        timeLimit: float | None = None,
        memoryLimit: int | None = None,
        **kwargs,
    ) -> Result:
        returnCode = 0
        stdout = None
        stderr = None
        with subprocess.Popen(*popenargs, **kwargs) as process:
            monitor = threading.Thread(
                target=self.memoryMonitor,
                args=(
                    process.pid,
                    timeLimit,
                    memoryLimit,
                ),
            )
            monitor.start()
            stdout, stderr = process.communicate(inputContent) # type: ignore
            # try:
            #     stdout, stderr = process.communicate(inputContent, timeout=timeLimit)
            # except subprocess.TimeoutExpired:
            #     process.kill()
            #     if mswindows():
            #         stdout, stderr = process.communicate()
            #     else:
            #         process.wait()
            #     timeOut = True
            returnCode = process.poll()
        return self.Result(self.timeOut, self.memoryOut, returnCode, stdout, stderr) # type: ignore


def checkDirectoryExists(directory: str) -> None:
    if not os.path.exists(directory):
        os.makedirs(directory)


def mswindows() -> bool:
    try:
        import msvcrt
    except ModuleNotFoundError:
        return False
    else:
        return True



from autohack.core.exception import *
from autohack.core.util import *
import subprocess


def compileCode(compileCommand: str, fileName: str) -> None:
    try:
        process = subprocess.Popen(
            compileCommand, stdout=subprocess.PIPE, stderr=subprocess.STDOUT
        )
    except OSError:
        return
    output = process.communicate()[0]
    if process.returncode != 0:
        raise CompilationError(fileName, output, process.returncode)


def generateInput(generateCommand: str, clientID: str) -> bytes:
    try:
        process = subprocess.Popen(
            generateCommand, stdout=subprocess.PIPE, stderr=subprocess.DEVNULL
        )
    except OSError:
        return b""
    dataInput = process.communicate()[0]
    if process.returncode != 0:
        raise InputGenerationError(dataInput, clientID, process.returncode)
    return dataInput


def generateAnswer(generateCommand: str, dataInput: bytes, clientID: str) -> bytes:
    try:
        process = subprocess.Popen(
            generateCommand,
            stdin=subprocess.PIPE,
            stdout=subprocess.PIPE,
            stderr=subprocess.DEVNULL,
        )
    except OSError:
        return b""
    dataAnswer = process.communicate(dataInput)[0]
    if process.returncode != 0:
        raise AnswerGenerationError(dataInput, dataAnswer, clientID, process.returncode)
    return dataAnswer


def runSourceCode(
    runCommand: str, dataInput: bytes, timeLimit: float | None, memoryLimit: int | None
) -> CodeRunner.Result:
    try:
        result = CodeRunner().run(
            runCommand,
            inputContent=dataInput,
            timeLimit=timeLimit,
            memoryLimit=memoryLimit,
            stdin=subprocess.PIPE,
            stdout=subprocess.PIPE,
            stderr=subprocess.DEVNULL,
        )
    except OSError:
        return CodeRunner.Result(False, False, 0, b"", b"")
    return result



from autohack.core.util import *
import os


class CompilationError(Exception):
    def __init__(self, fileName: str, message: bytes, returnCode: int) -> None:
        self.fileName = fileName
        self.message = message
        self.returnCode = returnCode

    def __str__(self) -> str:
        return f"\x1b[1;31m{self.fileName.capitalize()} compilation failed with return code {self.returnCode}.\x1b[0m\n\n{self.message.decode()}"


class InputGenerationError(Exception):
    def __init__(self, dataInput: bytes, clientID: str, returnCode: int) -> None:
        self.clientID = clientID
        self.returnCode = returnCode
        checkDirectoryExists(os.path.dirname(getTempInputFilePath(clientID)))
        open(getTempInputFilePath(clientID), "wb").write(dataInput)

    def __str__(self) -> str:
        return f"\x1b[1;31mInput generation failed with return code {self.returnCode}. Input saved to {getTempInputFilePath(self.clientID)}.\x1b[0m"


class AnswerGenerationError(Exception):
    def __init__(
        self, dataInput: bytes, dataAnswer: bytes, clientID: str, returnCode: int
    ) -> None:
        self.clientID = clientID
        self.returnCode = returnCode
        checkDirectoryExists(os.path.dirname(getTempInputFilePath(clientID)))
        checkDirectoryExists(os.path.dirname(getTempAnswerFilePath(clientID)))
        open(getTempInputFilePath(clientID), "wb").write(dataInput)
        open(getTempAnswerFilePath(clientID), "wb").write(dataAnswer)

    def __str__(self) -> str:
        return f"\x1b[1;31mAnswer generation failed with return code {self.returnCode}. Input saved to {getTempInputFilePath(self.clientID)}. Answer saved to {getTempAnswerFilePath(self.clientID)}.\x1b[0m"



import platformdirs, os

dirs = platformdirs.PlatformDirs("autohack", "Gavin", version="v1")

DATA_FOLDER_PATH = os.path.join(os.getcwd(), ".autohack")

RECORD_FILE_PATH = os.path.join(DATA_FOLDER_PATH, "record.txt")

LOG_FOLDER_PATH = os.path.join(DATA_FOLDER_PATH, "logs")
# LOG_FOLDER_PATH = dirs.user_log_dir

TEMP_FOLDER_PATH = dirs.user_runtime_dir

CONFIG_FILE_PATH = os.path.join(DATA_FOLDER_PATH, "config.json")

CURRENT_HACK_DATA_FOLDER_PATH = os.path.join(DATA_FOLDER_PATH, "hackdata")

HACK_DATA_STORAGE_FOLDER_PATH = os.path.join(DATA_FOLDER_PATH, "datastorage")


def getTempInputFilePath(clientID: str) -> str:
    return os.path.join(TEMP_FOLDER_PATH, clientID, "input")


def getTempAnswerFilePath(clientID: str) -> str:
    return os.path.join(TEMP_FOLDER_PATH, clientID, "answer")


def getHackDataStorageFolderPath(clientID: str) -> str:
    return os.path.join(HACK_DATA_STORAGE_FOLDER_PATH, clientID)


def getHackDataFilePath(dataID: int, filePath: str) -> str:
    return os.path.join(
        CURRENT_HACK_DATA_FOLDER_PATH, filePath.replace("$(id)", str(dataID))
    )


def getHackDataFolderPath(dataID: int, filePath: str) -> str:
    return os.path.dirname(getHackDataFilePath(dataID, filePath))
